==> requests2-0.1.31.crate: requests2-0.1.31/tests/css_selector/mod.rs <==
use dbfile::DBStore;
use dbfile_derive::DBfile;
use lazy_static::__Deref;
use requests2::{Cache, Headers, Requests, Store, Value};
use select::predicate::{Class, Name};
use std::{collections::HashMap, rc::Rc, thread, time::Duration};

pub fn run_selector() {
    // .class .class.class #[attr=value] #id
    let cache = Cache::new();

    let rq2 = Requests::new(&cache).connect("https://news.163.com/", Headers::Default);
    rq2.parser(
        |mut p| {
            p.select(".news_default_news", |n| {
                let data: Vec<String> = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            })
        },
        "163-news",
    );

    rq2.parser(
        |mut p| {
            p.select(".ns_area.list", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            })
        },
        "163-tab",
    );

    cache.print_json();

    let rq = Requests::new(&cache).connect("https://www.qq.com", Headers::Default);
    rq.parser(
        |mut p| {
            let attr = p.select("[bosszone=th_1]", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            });

            let class1_class2 = p.select(".tit.fl", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            });

            let id = p.select("#searchTab", |n| {
                let data = n
                    .into_iter()
                    .map(|n| n.text().split_ascii_whitespace().collect())
                    .collect::<Vec<String>>();
                Value::STR(data[0].clone())
            });
            let map = HashMap::from([
                ("attr".to_owned(), attr),
                ("class_class".to_owned(), class1_class2),
                ("id".to_owned(), id),
            ]);
            Value::MAP(map)
        },
        "qq",
    );

    cache.print();
    let _value = cache.get("qq");
    // println!("Value: {:?}", value);
}

pub fn run_parser_select() {
    let data = Cache::new();
    let client = Requests::new(&data);
    let parser = client.connect("https://www.qq.com", Headers::Default);
    parser.parser(
        |mut p| {
            let mut result = HashMap::new();

            let navs = p.select("li.nav-item", |nodes| {
                let navs = nodes
                    .into_iter()
                    .map(|n| {
                        let mut item = HashMap::new();
                        n.find(Name("a")).next().map_or(
                            HashMap::from([("".to_string(), Value::NULL)]),
                            |a| {
                                let nav_name = a.text();
                                let nav_href =
                                    a.attr("href").map_or(String::from(""), |x| x.to_string());
                                item.insert("nav_name".to_string(), Value::STR(nav_name));
                                item.insert("nav_href".to_string(), Value::STR(nav_href));
                                item
                            },
                        )
                    })
                    .collect::<Vec<HashMap<String, Value>>>();

                Value::VECMAP(navs)
            });

            let news = p.select("ul.yw-list", |nodes| {
                let mut news = Vec::new();

                for node in nodes {
                    for n in node.find(Class("news-top")) {
                        for a in n.find(Name("a")) {
                            let title = a.text();
                            news.push(title);
                        }
                    }
                }

                Value::LIST(news)
            });

            result.insert("titles".to_owned(), news);
            result.insert("nav".to_owned(), navs);

            Value::MAP(result)
        },
        "index",
    );

    data.print();
}

pub fn run_free_parse_and_store_to_csv() {
    let data = Cache::new();
    let client = Requests::new(&data);
    let rq = client.connect("https://www.qq.com/", Headers::Default);

    #[derive(DBfile, Debug)]
    struct Link<'a> {
        href: &'a str,
        link_name: String,
        title: &'a str,
    }

    rq.free_parse(|mut p| {
        let mut title = String::new();

        p.free_select("title", |n| title = n[0].text());

        p.free_select("li.nav-item a", |n| {
            let links = n
                .iter()
                .map(|x| Link {
                    title: &title,
                    href: x.attr("href").expect("extra href error"),
                    link_name: x.text(),
                })
                .collect::<Vec<Link>>();

            DBStore::to_csv(links, "D:\\links.csv", "a", true);
        });
    });
}

pub fn crawl_kongfz() {
    let mut urls = Vec::new();

    for p in 200..201 {
        let mut _hash = HashMap::new();
        _hash.insert("url", format!("https://book.kongfz.com/Cxianzhuang/cat_8001w{}/", p));
        _hash.insert("page", format!("{}", p));
        urls.push(_hash);
    }

    for url in urls {
        let p:i32 = url.get("page").unwrap().parse().unwrap();
        crawl_kongfz_by_page(p, url.get("url").unwrap().to_string());
        thread::sleep(Duration::from_millis(2000));
    }
}

pub fn crawl_kongfz_by_page(page: i32, url: String) {
    let data = Cache::new();
    let client = Requests::new(&data);
    let rq = client.connect(&url, Headers::Default);

    #[derive(DBfile, Debug)]
    struct Item<'a> {
        book_name: &'a str,
        price: f32,
        href: &'a str,
        img:&'a str,
        isbn:&'a str,
    }

    rq.free_parse(|mut p| {
        p.free_select(".item.clearfix", |nodes| {
            let book_item = nodes
                .iter()
                .enumerate()
                .filter(|(_idx, n)| n.attr("itemname").is_some())
                .map(|(idx, x)| {

                    let book_name = Rc::new(x.attr("itemname").unwrap().to_string());
                    let isbn = Rc::new(x.attr("isbn").unwrap().to_string());
                    let price:f32 = Rc::new(x.attr("price").unwrap()).parse().unwrap();
                    let href = Rc::new(x.find(Name("a")).find_map(|n| n.attr("href")).unwrap().to_string());
                    let img = Rc::new(x.find(Name("img")).find_map(|n| n.attr("src")).unwrap().to_string());
                    
                    
                    let mut header = false;
                    if idx == 0 && page == 1 { header = true };
                    DBStore::to_csv(vec![
                        Item { 
                            book_name: Rc::clone(&book_name).deref(),
                            price,
                            href: Rc::clone(&href).deref(),
                            img: Rc::clone(&img).deref(),
                            isbn: Rc::clone(&isbn).deref(),
                        }
                    ], "links2.csv", "a", header);

                    let hash = HashMap::from([
                        ("book_name".to_string(), Rc::clone(&book_name)),
                        ("href".to_string(), Rc::clone(&href)),
                        ("isbn".to_string(), Rc::clone(&isbn)),
                        // ("price".to_string(), Rc::new(price.to_string())),
                        ("img".to_string(), Rc::clone(&img))
                        ]);
                    // }
                    hash
                })
                .collect::<Vec<HashMap<String, Rc<String>>>>();

            print!("{:?}", book_item)
        });
    })
}

==> requests2-0.1.42.crate: requests2-0.1.42/tests/css_selector/mod.rs <==
use dbfile::DBfile;
use dbfile_derive::{DBfile, dbnote};
use requests2::{Cache, Headers, Requests, Store, Value};
use select::predicate::{Class, Name};
use std::{collections::HashMap};

pub fn run_selector() {
    // .class .class.class #[attr=value] #id
    let cache = Cache::new();

    let rq2 = Requests::new(&cache).connect("https://news.163.com/", Headers::Default);
    rq2.parser(
        |mut p| {
            p.select(".news_default_news", |n| {
                let data: Vec<String> = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            })
        },
        "163-news",
    );

    rq2.parser(
        |mut p| {
            p.select(".ns_area.list", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            })
        },
        "163-tab",
    );

    cache.print_json();

    let rq = Requests::new(&cache).connect("https://www.qq.com", Headers::Default);
    rq.parser(
        |mut p| {
            let attr = p.select("[bosszone=th_1]", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            });

            let class1_class2 = p.select(".tit.fl", |n| {
                let data = n
                    .iter()
                    .map(|n| n.text().split_whitespace().collect())
                    .collect();
                Value::LIST(data)
            });

            let id = p.select("#searchTab", |n| {
                let data = n
                    .into_iter()
                    .map(|n| n.text().split_ascii_whitespace().collect())
                    .collect::<Vec<String>>();
                Value::STR(data[0].clone())
            });
            let map = HashMap::from([
                ("attr".to_owned(), attr),
                ("class_class".to_owned(), class1_class2),
                ("id".to_owned(), id),
            ]);
            Value::MAP(map)
        },
        "qq",
    );

    cache.print();
    let _value = cache.get("qq");
    // println!("Value: {:?}", value);
}

pub fn run_parser_select() {
    let data = Cache::new();
    let client = Requests::new(&data);
    let parser = client.connect("https://www.qq.com", Headers::Default);
    parser.parser(
        |mut p| {
            let mut result = HashMap::new();

            let navs = p.select("li.nav-item", |nodes| {
                let navs = nodes
                    .into_iter()
                    .map(|n| {
                        let mut item = HashMap::new();
                        n.find(Name("a")).next().map_or(
                            HashMap::from([("".to_string(), Value::NULL)]),
                            |a| {
                                let nav_name = a.text();
                                let nav_href =
                                    a.attr("href").map_or(String::from(""), |x| x.to_string());
                                item.insert("nav_name".to_string(), Value::STR(nav_name));
                                item.insert("nav_href".to_string(), Value::STR(nav_href));
                                item
                            },
                        )
                    })
                    .collect::<Vec<HashMap<String, Value>>>();

                Value::VECMAP(navs)
            });

            let news = p.select("ul.yw-list", |nodes| {
                let mut news = Vec::new();

                for node in nodes {
                    for n in node.find(Class("news-top")) {
                        for a in n.find(Name("a")) {
                            let title = a.text();
                            news.push(title);
                        }
                    }
                }

                Value::LIST(news)
            });

            result.insert("titles".to_owned(), news);
            result.insert("nav".to_owned(), navs);

            Value::MAP(result)
        },
        "index",
    );

    data.print();
}

pub fn run_free_parse_and_store_to_csv() {
    let data = Cache::new();
    let client = Requests::new(&data);
    let rq = client.connect("https://www.qq.com/", Headers::Default);

    #[derive(DBfile, Debug)]
    #[dbnote(table_name = "test_link.csv")]
    struct Link<'a> {
        href: &'a str,
        link_name: String,
        title: &'a str,
    }

    rq.free_parse(|mut p| {
        let mut title = String::new();

        p.free_select("title", |n| title = n[0].text());

        p.free_select("li.nav-item a", |n| {
            let links = n
                .iter()
                .map(|x| Link {
                    title: &title,
                    href: x.attr("href").unwrap_or_default(),
                    link_name: x.text(),
                })
                .collect::<Vec<Link>>();
            
            for (idx, link) in links.into_iter().enumerate() {
                if idx == 0 {
                    link.write_csv_head();
                }
                link.to_csv("a");
            }
        });
    });
}
